Package: cellWise
Type: Package
Version: 2.5.7
Date: 2026-02-25
Title: Analyzing Data with Cellwise Outliers
Authors@R: c(
            person(given="Jakob",family="Raymaekers", role = c("aut","cre"),
            email="jakob.raymaekers@uantwerpen.be"),
            person(given="Peter",family="Rousseeuw", role = c("aut")),
            person(given="Wannes",family="Van den Bossche", role = c("ctb")),
            person(given="Mia",family="Hubert", role = c("ctb")) 
             )
Depends: R (>= 4.0.0)
Suggests: knitr, robustHD, MASS, ellipse, markdown, rospca, GSE
Imports: reshape2, scales, ggplot2, matrixStats, gridExtra, robustbase,
        rrcov, svd, stats, utils, shape, Rcpp (>= 0.12.10.14)
LinkingTo: Rcpp, RcppArmadillo (>= 0.7.600.1.0)
Description: Tools for detecting cellwise outliers and robust methods to analyze
    data which may contain them. Contains the implementation of the algorithms described in
    Rousseeuw and Van den Bossche (2018) <doi:10.1080/00401706.2017.1340909> (open access)
    Hubert et al. (2019) <doi:10.1080/00401706.2018.1562989> (open access),
    Raymaekers and Rousseeuw (2021) <doi:10.1080/00401706.2019.1677270> (open access),
    Raymaekers and Rousseeuw (2021) <doi:10.1007/s10994-021-05960-5> (open access),
    Raymaekers and Rousseeuw (2021) <doi:10.52933/jdssv.v1i3.18> (open access),
    Raymaekers and Rousseeuw (2022) <doi:10.1080/01621459.2023.2267777> (open access)
    Rousseeuw (2022) <doi:10.1016/j.ecosta.2023.01.007> (open access).
    Examples can be found in the vignettes:
    "DDC_examples", "MacroPCA_examples", "wrap_examples", "transfo_examples",
    "DI_examples", "cellMCD_examples" , "Correspondence_analysis_examples",
    and "cellwise_weights_examples".
License: GPL (>= 2)
LazyData: No
Author: Jakob Raymaekers [aut, cre],
  Peter Rousseeuw [aut],
  Wannes Van den Bossche [ctb],
  Mia Hubert [ctb]
Maintainer: Jakob Raymaekers <jakob.raymaekers@uantwerpen.be>
VignetteBuilder: knitr
RoxygenNote: 7.1.2
NeedsCompilation: yes
Packaged: 2026-02-25 21:23:51 UTC; Jakob Raymaekers
Repository: CRAN
Date/Publication: 2026-02-26 06:10:27 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 06:23:00 UTC; unix
Archs: cellWise.so.dSYM
